@@ -1,6 +1,5 @@
 import contextlib
 import functools
-import inspect
 from io import StringIO
 import os.path as osp
 
@@ -9,7 +8,8 @@ from dash.exceptions import PreventUpdate
 from ansi2html import Ansi2HTMLConverter
 import visdcc
 
-from oneface.types import (Selection, SubSet, InputPath, OutputPath)
+from .types import (Selection, SubSet, InputPath, OutputPath)
+from .arg import Empty, get_func_argobjs
 
 
 HERE = osp.abspath(osp.dirname(__file__))
@@ -28,14 +28,19 @@ class App(object):
     def __init__(
             self, func, name=None,
             show_console=True,
-            conosole_interval=2000,
+            console_interval=2000,
             interactive=False, init_run=False,
             result_show_type="text",
             **server_args):
         self.func = func
-        self.name = name
+        if name is not None:
+            self.name = name
+        elif hasattr(func, "name"):
+            self.name = func.name
+        else:
+            self.name = func.__name__
         self.show_console = show_console
-        self.console_interval = conosole_interval
+        self.console_interval = console_interval
         self.interactive = interactive
         self.init_run = init_run
         self.result_show_type = result_show_type
@@ -102,21 +107,19 @@ class App(object):
         return layout
 
     def parse_args(self):
-        from oneface.core import Arg
-        sig = inspect.signature(self.func)
         widgets, names, types, attrs = [], [], [], []
-        for n, p in sig.parameters.items():
-            ann = p.annotation
-            if not isinstance(ann, Arg):
+        arg_objs = get_func_argobjs(self.func)
+        for n, a in arg_objs.items():
+            if a.type is Empty:
                 continue
-            constructor = self.type_to_widget_constructor[ann.type.__name__]
-            default = None if p.default is inspect._empty else p.default
-            attr = ann.kwargs
+            constructor = self.type_to_widget_constructor[a.type.__name__]
+            default = None if a.default is Empty else a.default
+            attr = a.kwargs
             widget = constructor(
-                n, ann.range, default, attrs=attr).widget
+                n, a.range, default, attrs=attr).widget
             widgets.append(widget)
             names.append(n)
-            types.append(ann.type)
+            types.append(a.type)
             attrs.append(attr)
         return widgets, names, types, attrs
 
@@ -137,10 +140,12 @@ class App(object):
         deco = app.callback(output, *inputs)
         return deco
 
-    def get_dash_app(self):
+    def get_dash_app(self, *args, **kwargs):
         name = self.name or self.func.__name__
         css = ['https://codepen.io/chriddyp/pen/bWLwgP.css']
-        app = Dash(name, external_stylesheets=css)
+        if 'external_stylesheets' not in kwargs:
+            kwargs['external_stylesheets'] = css
+        app = Dash(name, *args, **kwargs)
         app.layout = self.get_layout()
         console_buffer = StringIO()
 
@@ -244,9 +249,11 @@ class InputItem(object):
 
 class IntInputItem(InputItem):
     def get_input(self):
+        _range = self.range or [0, 100]
+        _default = _range[0] if (self.default is None) else self.default
         return dcc.Input(
-            min=self.range[0], max=self.range[1], type="number", step=1,
-            value=(self.default or self.range[0]), style={
+            min=_range[0], max=_range[1], type="number", step=1,
+            value=_default, style={
                 'width': "100%",
             }
         )
@@ -254,9 +261,11 @@ class IntInputItem(InputItem):
 
 class FloatInputItem(InputItem):
     def get_input(self):
+        _range = self.range or [0, 100]
+        _default = _range[0] if (self.default is None) else self.default
         return dcc.Slider(
-            self.range[0], self.range[1], step=None,
-            value=(self.default or self.range[0])
+            _range[0], _range[1], step=None,
+            value=_default,
         )
 
 
@@ -277,9 +286,10 @@ class StrInputItem(InputItem):
 
 class BoolInputItem(InputItem):
     def get_input(self):
+        _default = True if self.default is None else self.default
         return dcc.RadioItems(
             ["True", "False"],
-            value=(str(self.default) or "True")
+            value=str(_default)
         )
 
 
@@ -287,7 +297,7 @@ class DropdownInputItem(InputItem):
     def get_input(self):
         return dcc.Dropdown(
             self.range,
-            value=(self.default or range[0])
+            value=(self.default or self.range[0])
         )
 
 
@@ -295,7 +305,7 @@ class MultiDropdownInputItem(InputItem):
     def get_input(self):
         return dcc.Dropdown(
             self.range,
-            value=(self.default or range[0]), multi=True
+            value=(self.default or self.range[0]), multi=True
         )
 
 
